/**
 * \file	DtcpCTcpSocket.h
 * \brief	DtcpCTcpSocket class
 *
 * - Copyright : Samsung Electronics CO.LTD.,
 *
 * \internal
 * Author : Soyoung Lee, Sangsu Choi
 * Dept : Core S/W Component Lab, Digital Media Laboratory
 * Creation date : 2008/09/02 
 */

#ifndef __DTCPCTCPSOCKET_H
#define __DTCPCTCPSOCKET_H


#ifdef _WIN32_ 
// <windows.h> <winsock.h> includeϰ 
// common C++ <winsock2.h> includeؼ struct redefinition problem ߻
// ϹǷ  literal ߰
#define _WINSOCKAPI_ 
        #include <windows.h>
        #include <winsock2.h>
        #include <windef.h>
#elif defined _ANYVIEW
// <windows.h> <winsock.h> includeϰ 
// common C++ <winsock2.h> includeؼ struct redefinition problem ߻
// ϹǷ  literal ߰
#define _WINSOCKAPI_ 
        #include <windows.h>
        #include <winsock2.h>
        #include <windef.h>
#elif  _LINUX  
        #include <unistd.h>
        #include <sys/socket.h>
		#include <sys/ioctl.h>
        #include <arpa/inet.h>
        #include <fcntl.h>
#endif
#include <cstdio>

#include "CC_Type.h"
//#include "DtcpCConstants.h"
//#include "DtcpCErrorCode.h"

////////////////////////////////////////////////////////////////////////////////
// Constants
////////////////////////////////////////////////////////////////////////////////
#define DTCP_QUE_LIMIT		5		// Maximum outstanding connection requests
#define INFINITE_32			0xffffffff

/**
* DtcpCTcpSocket class
*/
class DtcpCTcpSocket  
{
public:

#ifndef _DTCP_PLUS
	/**
	constructor
	*/
	DtcpCTcpSocket();
	/**
	constructor 
	@param uFd (Input) - File descriptor
	*/
	DtcpCTcpSocket(cc_u32 uFd);

#else
	/**
	constructor
	*/
	DtcpCTcpSocket(bool bSetTTL = true);
	/**
	constructor 
	@param uFd (Input) - File descriptor
	*/
	DtcpCTcpSocket(cc_u32 uFd, bool bSetTTL = true);
#endif

	/**
	destructor
	*/
	virtual ~DtcpCTcpSocket();

	/**
	Bind  
	@param pAddr (Input) - sockaddr structure  
	@return Errorcode
	*/
	int					bindSocket(sockaddr_in* pAddr);

	/**
	Connect Լ  
	@param pDest (Input) - sockaddr structure  
	*/
	long				connectSocket(sockaddr_in* pDest);

	/**
	close socket Լ 
	@return Errorcode
	*/
	int					closeSocket(void);

	/**
	TTL  
	*/
#ifdef _DTCP_PLUS
	void			SetTTL (bool bSetTTL = true);
#else
	void			SetTTL ();
#endif
	
	/**
	read method
	@param	pbyBuf (Output)-  ޽ pointer
	@param	dwBufLen (Input)-  ũ 
	@param	pdwRcvLen (Output)-  ޽ ũ 
	@param	dwMsec (Input) - timeout ð 

	@return	Errorcode 		
	*/	
	int read(cc_u8* pbyBuf, 
			  cc_u32 dwBufLen, 
			  cc_u32* pdwRcvLen, 
			  cc_u32 dwMsec);		
		
	/*
	//read in non-blocking mode 
	inline cc_u32 readNB(char* pchBuf, 
					  cc_u32 dwBufLen, 
					  cc_u32* pdwRcvLen)
	{
		return read(pchBuf, dwBufLen, pdwRcvLen, 0);
	};
	
	  */
	
	// query TCP socket to see how many bytes are in read buffer
	/**
	read bufferȿ  Ʈ ִ Ȯ
	@param iBufLen (Output)-  ޽ bytes
	@return Errorcode
	*/
	long readBufLen(int* iBufLen);

	/**	

	@param	pchBuf (Input) -  ޽ 
	@param	dwBufLen (Input) -  ޽  
	@param	pdwSndLen (Output) -  ޽  

	@return	Errorcode
	*/
	int send(const char* pchBuf, 
			 cc_u32 dwBufLen, 
			 cc_u32* pdwSndLen);

	/*int send(const cc_u8* pbyBuf, 
			 cc_u32 dwBufLen, 
			 cc_u32* pdwSndLen);

	*/

		
	/**
	file descriptor  
	@return file descriptor value
	*/
	inline int getFd(void) const {return m_iFd;};
		
	//////////////////////////////////////////////////////////////////////
	// Address(member attributes) interface functions
	/**
	local Address 
	@return address
	*/
	inline cc_u32			getLocalAddr(void) const {return ntohl(m_AddrLocal.sin_addr.s_addr);};

	/**
	local port host byte    
	@return port
	*/
	inline cc_u16	getLocalPort(void) const {return ntohs(m_AddrLocal.sin_port);};

	/**
	 host byte ּ  
	@return address
	*/
	inline cc_u32			getDestAddr(void) const {return ntohl(m_AddrDest.sin_addr.s_addr);};

	/**
	 host byte port  
	@return port
	*/
	inline cc_u16	getDestPort(void) const {return ntohs(m_AddrDest.sin_port);};
	
	/**
	local address string  
	@return address sting
	*/
	inline char* getLocalAddrStr(void) {return inet_ntoa(m_AddrLocal.sin_addr);};

	/**
	 address string  
	@return address string 
	*/
	inline char* getLocalDestStr(void) {return inet_ntoa(m_AddrDest.sin_addr);};

	//////////////////////////////////////////////////////////////////////
	// Non-blocking
	/**
	nonblocking  ˻
	@return nonblocking flag 
	*/
	inline bool isNonBlocking(void) const {return m_bNonBlocking;};

	/**
	nonblocking flag 
	@param bNonBlocking (Input) - nonbloking flag
	@return     
	*/
	bool 		setNonBlocking(bool bNonBlocking);

	/**
	listen Լ 
	@return Errorcode
	*/
	int 			listen(void);
	
	/**
	accept Լ 
	@return ο socket pointer
	*/
	DtcpCTcpSocket*	accept(void);
protected:	

private:
	/**
	socket    ʱȭ(file descriptor,address copy)
	@return Errorcode
	*/
	int		init(void);

//Variable
public:
protected:
private:
#ifdef _WIN32_
		SOCKET			m_iFd;
#elif  _ANYVIEW
		SOCKET			m_iFd;
#else
		int 			m_iFd;
#endif
		struct			sockaddr_in m_AddrLocal;	
		struct			sockaddr_in m_AddrDest;

		bool			m_bNonBlocking;
#ifdef _DTCP_PLUS		
		bool			m_bSetTTL; 
#endif
};

#endif 
